/*
     This file is part of GNUnet

     GNUnet is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published
     by the Free Software Foundation; either version 2, or (at your
     option) any later version.

     GNUnet is distributed in the hope that it will be useful, but
     WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
     General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with GNUnet; see the file COPYING.  If not, write to the
     Free Software Foundation, Inc., 59 Temple Place - Suite 330,
     Boston, MA 02111-1307, USA.
*/

/**
 * @file include/util/configuration.h
 * @author Christian Grothoff
 * 
 * Configuration management.
 **/

#ifndef CONFIGURATION_H
#define CONFIGURATION_H

#include "util/ipcheck.h"
#include "util/statuscalls.h"

/* ****************** config values **************** */

/**
 * Default name of the configuration file.
 **/
#define DEFAULT_CONFIG_FILE "~/.gnunet/gnunet.conf"

/* *******************API *********************** */

/**
 * This method must be called first. Typically,
 * the command line is parsed after that and then
 * "readConfiguration" is invoked.
 **/
void initConfiguration();

/**
 * Read the specified configuration file. The previous
 * configuration will be discarded if this method is 
 * invoked twice. The configuration file that is read
 * can be set using setConfigurationString on 
 * section "FILES" and option "gnunet.conf".
 * 
 * This method should be invoked after the options have
 * been parsed (and eventually the configuration filename
 * default has been overriden) and if gnunetd receives
 * a SIGHUP.
 **/
void readConfiguration();

/**
 * This method may be called at last to clean up.
 * Afterwards everything but initConfiguration will result
 * in errors...
 **/
void doneConfiguration();

/**
 * Obtain a filename from the given section and option.  If the
 * filename is not specified, die with the given error message (do not
 * die if errMsg == NULL). 
 *
 * @param section from which section, may not be NULL
 * @param option which option, may not be NULL
 *
 * @param errMsg the errormessage, should contain two %s tokens for
 * the section and the option.
 *
 * @return the specified filename (caller must free), or NULL if no
 * filename was specified and errMsg == NULL
 **/
char * getFileName(char * section,
		   char * option,
		   const char * errMsg);

/**
 * Check if a string in the configuration matches a given value.  This
 * method should be preferred over getConfigurationString since this
 * method can avoid making a copy of the configuration string that
 * then must be freed by the caller.
 *
 * @param section from which section, may not be NULL
 * @param option which option, may not be NULL
 * @param value the value to compare against
 * @return YES or NO
 **/
int testConfigurationString(char * section,
			    char * option,
			    const char * value);

/**
 * Obtain a string from the configuration.
 * @param section from which section, may not be NULL
 * @param option which option, may not be NULL
 * @return a freshly allocated string, caller must free!
 *   Note that the result can be NULL if the option is not set.
 **/
char * getConfigurationString(char * section,
			      char * option);

/**
 * Obtain an int from the configuration.
 * @param section from which section
 * @param option which option
 * @return 0 if no option is specified
 **/
unsigned int getConfigurationInt(char * section,
				 char * option);

/**
 * Set an option.
 * @param section from which section, may not be NULL
 * @param option which option, may not be NULL
 * @param value the value to use, may be NULL
 * @return the previous value (or NULL if none),
 *     caller must free!
 **/
char * setConfigurationString(char * section,
			      char * option,
			      const char * value);

/**
 * Set an option.
 * @param section from which section, may not be NULL
 * @param option which option, may not be NULL
 * @param value the value to use
 * @return the previous value (or 0 if none)
 **/
unsigned int setConfigurationInt(char * section,
				 char * option,
				 const unsigned int value);

/**
 * Get the command line strings (the ones
 * remaining after getopt-style parsing).
 * @param value the values
 + @return the number of values
 **/
int getConfigurationStringList(char *** value);

/**
 * Set the list of command line options (remainder
 * after getopt style parsing).
 * @param value the values 
 + @param count the number of values
 **/
void setConfigurationStringList(char ** value,
				int count);

#endif
